#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>	/* inet_aton,inet_ntoa */
#include <signal.h>	/* signal,SIG* */
#include <errno.h>	/* errno,E* */
#include <unistd.h>	/* read,write,close */
#include <string.h>	/* memset */
#include <stdio.h>	/* FILE,fprintf */

#define UDP_DEBUG_DST_PORT 0x2021
#define UDP_DEBUG_SRC_PORT 1025

#define DEBUG_BUFFER_SIZE 1024	// FIXME: check the real size

// FIXME: see Palm1DebugConvert.cpp

void human_debug(const char *buffer, int size) {
	const unsigned char *p;
	for (p = (unsigned char *)buffer; *p; p++) {
		if (32 <= *p && *p <= 127) {
			fprintf(stdout, "%c", *p);
		} else if (*p == '\r' && *(p+1) == '\n') {
			fprintf(stdout, "\\n");
			p++;
		} else {
			fprintf(stdout, "\\%03o", *p);
		}
	}
}

int main(int argc, const char *argv[]) {
	const char *progname;
	int sd;
	struct sockaddr_in sin;

	// check arguments count
	progname = argv[0];
	if (argc != 1) {
		fprintf(stderr, "Syntax: %s\n", progname);
		return 1;
	}

	// prepare receiving answers (bind debug port)
	sd = socket(PF_INET, SOCK_DGRAM, 0);
	if (sd < 0) {
		fprintf(stderr, "Unable to create a socket!\n");
		return 3;
	}
	memset(&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(UDP_DEBUG_DST_PORT);
	sin.sin_addr.s_addr = htonl(INADDR_ANY);
	if (bind(sd, (struct sockaddr *)&sin, sizeof(sin))) {
		fprintf(stderr, "Unable to bind!\n");
		return 4;
	}

	// wait for answers
	signal(SIGPIPE, SIG_IGN);
	while (1) {
		char buffer[DEBUG_BUFFER_SIZE];
		struct sockaddr_in sinrecv;
		socklen_t sinrecv_len = sizeof(sinrecv);
		int size = recvfrom(sd, buffer, sizeof(buffer), 0, (struct sockaddr *)&sinrecv, &sinrecv_len);
		if (size < DEBUG_BUFFER_SIZE) {
			fprintf(stdout, "%s:", inet_ntoa(sinrecv.sin_addr));
			buffer[size] = '\0'; // FIXME: should be managed below
			human_debug(buffer, size);
			fprintf(stdout, "\n");
			fflush(stdout);
		} else {
			fprintf(stderr, "Warning: received an invalid packet size of %d bytes!\n", size);
		}
	}

	// cleaning
	close(sd);
	return 0;
}
