#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>	/* inet_aton,inet_ntoa */
#include <unistd.h>	/* read,write,close */
#include <string.h>	/* memset */
#include <stdio.h>	/* FILE,fprintf */

#define UDP_DEBUG_SRC_PORT 1025
#define REPORT_DST_IPADDR "224.1.2.5"

int main(int argc, const char *argv[]) {
	const char *progname, *message, *ipaddr;
	struct in_addr sin_addr;
	int sd;
	struct sockaddr_in sin;

	// check arguments count
	progname = argv[0];
	if (argc == 2) { // multicast
		message = argv[1];
		ipaddr = REPORT_DST_IPADDR;
	} else if (argc == 3) { // specific destination
		message = argv[1];
		ipaddr = argv[2];
	} else {
		fprintf(stderr, "Syntax: %s <message> [<IPv4-address>]\n", progname);
		return 1;
	}
	setbuf(stdout, NULL);

	// check IPv4 address
	if (!inet_aton(ipaddr, &sin_addr)) {
		fprintf(stderr, "'%s' is an invalid IPv4 address!\n", ipaddr);
		return 2;
	}
	fprintf(stdout, "IPv4 address: %s\n", inet_ntoa(sin_addr));

	// prepare receiving answers (bind report port)
	sd = socket(PF_INET, SOCK_DGRAM, 0);
	if (sd < 0) {
		fprintf(stderr, "Unable to create a socket!\n");
		return 3;
	}

	// send message to display
	memset(&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(UDP_DEBUG_SRC_PORT);
	inet_aton(ipaddr, &sin.sin_addr);
	if (sendto(sd, message, strlen(message)+1, 0, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		fprintf(stderr, "Error sending UDP request!\n");
		return 4;
	}

	// cleaning
	close(sd);
	return 0;
}
