#include <unistd.h>	/* read,write,close */
#include <string.h>	/* str* */
#include <stdlib.h>	/* atoi */
#include <stdio.h>	/* FILE,fprintf */

#define SETTINGS_HINT "sh"

#define TOTAL_SETTINGS_SIZE 4096

#define SPEED_DIAL_BEGIN 0x0200
#define SPEED_DIAL_NUM 100
#define SPEED_DIAL_LEN 32
#define PHONE_NUMBER_LEN 16

void display_phonebook(FILE *filehandle, int start, int end) {
	char buffer[SPEED_DIAL_LEN*SPEED_DIAL_NUM], *pos;
	if (start < 0) start = 0;
	if (end > (SPEED_DIAL_NUM - 1)) end = SPEED_DIAL_NUM - 1;
	fseek(filehandle, SPEED_DIAL_BEGIN + (SPEED_DIAL_LEN * start), SEEK_SET);
	pos = buffer + (SPEED_DIAL_LEN * start);
	fread(pos, SPEED_DIAL_LEN, (end - start + 1), filehandle);
	fprintf(stdout, "#index:number:name\n");
	while (start <= end) {
		// FIXME: should not be ".16" but something like ".%d",PHONE_NUMBER_LEN
		fprintf(stdout, "%d:%.16s:%.16s\n", start, pos, pos + PHONE_NUMBER_LEN);
		pos += SPEED_DIAL_LEN;
		start++;
	}
}

int change_phonebook(FILE *filehandle, int index, const char *number, const char *name) {
	char buffer[SPEED_DIAL_LEN];
	if (index < 0 || (SPEED_DIAL_NUM - 1) < index) return -1;
	strncpy(buffer, number, PHONE_NUMBER_LEN);
	strncpy(buffer + PHONE_NUMBER_LEN, name, SPEED_DIAL_LEN - PHONE_NUMBER_LEN);
	fseek(filehandle, SPEED_DIAL_BEGIN + (SPEED_DIAL_LEN * index), SEEK_SET);
	if (fwrite(buffer, SPEED_DIAL_LEN, 1, filehandle) != 1) return -2;
	return 0;
}

int main(int argc, const char *argv[]) {
	const char *progname, *filename, *extension;
	FILE *filehandle;
	size_t length;

	// check arguments count
	progname = argv[0];
	if (argc < 2 || 5 < argc) {
		fprintf(stderr, "Syntax: %s <filename> [<index>[-<end>] [<new-number> [<new-name>]]]\n", progname);
		return 1;
	}
	filename = argv[1];

	// check first argument (filename)
	filehandle = fopen(filename, "r+b");
	if (filehandle == NULL) {
		fprintf(stderr, "Unable to open '%s' !\n", filename);
		return 2;
	}
	// fprintf(stdout, "# File name: %s\n", filename);

	// check file type (by extension, for now...)
	extension = strrchr(filename, '.');
	if (extension && strcasecmp(extension, ".sh")) {
		fprintf(stderr, "Unknown file type!\n");
		return 2;
	}

	// check file size
	fseek(filehandle, 0L, SEEK_END);
	length = ftell(filehandle);
	if (length != TOTAL_SETTINGS_SIZE) {
		fprintf(stderr, "Invalid file size (%d != %d) !\n", length, TOTAL_SETTINGS_SIZE);
		return 3;
	}
	// fprintf(stdout, "# File length: %d bytes\n", length);

	// proceed as requested
	if (argc < 4) {
		int start, end;
		if (argc > 2) {
			char *pos, tmp[8]; /* "NNN-NNN\0" */
			strncpy(tmp, argv[2], sizeof(tmp));
			tmp[sizeof(tmp)-1] = '\0';
			pos = strchr(tmp, '-');
			if (pos) {
				*(pos++) = 0;
				start = atoi(tmp);
				end = *pos ? atoi(pos) : (SPEED_DIAL_NUM - 1);
			} else {
				start = end = atoi(tmp);
			}
		} else {
			start = 0;
			end = SPEED_DIAL_NUM - 1;
		}
		display_phonebook(filehandle, start, end);
	} else {
		int index;
		const char *number, *name;
		index = atoi(argv[2]);
		number = argv[3];
		name = (argc >= 5) ? argv[4] : "";
		change_phonebook(filehandle, index, number, name);
	}

	// cleaning
	fclose(filehandle);
	return 0;
}
